@@ -25,6 +25,7 @@
 #include "auth.h"
 #include "libcli/security/security.h"
 #include "lib/util/tevent_ntstatus.h"
+#include "source3/lib/substitute.h"
 
 NTSTATUS auth3_generate_session_info(struct auth4_context *auth_context,
 				     TALLOC_CTX *mem_ctx,
@@ -57,7 +58,7 @@ NTSTATUS auth3_generate_session_info(struct auth4_context *auth_context,
 		if (user_info->num_sids != 1) {
 			return NT_STATUS_INTERNAL_ERROR;
 		}
-		sid = &user_info->sids[PRIMARY_USER_SID_INDEX];
+		sid = &user_info->sids[PRIMARY_USER_SID_INDEX].sid;
 
 		cmp = dom_sid_compare(sid, &global_sid_System);
 		if (cmp == 0) {
