@@ -75,7 +75,7 @@ NTSTATUS vfs_default_durable_cookie(struct files_struct *fsp,
 		return NT_STATUS_NOT_SUPPORTED;
 	}
 
-	if (is_ntfs_stream_smb_fname(fsp->fsp_name)) {
+	if (fsp_is_alternate_stream(fsp)) {
 		/*
 		 * We do not support durable handles
 		 * on streams for now.
@@ -486,14 +486,14 @@ static bool vfs_default_durable_reconnect_check_stat(
 	}
 
 	if (cookie_st->st_ex_flags != fsp_st->st_ex_flags) {
-		DEBUG(1, ("vfs_default_durable_reconnect (%s): "
-			  "stat_ex.%s differs: "
-			  "cookie:%llu != stat:%llu, "
-			  "denying durable reconnect\n",
-			  name,
-			  "st_ex_flags",
-			  (unsigned long long)cookie_st->st_ex_flags,
-			  (unsigned long long)fsp_st->st_ex_flags));
+		DBG_WARNING(" (%s): "
+			    "stat_ex.%s differs: "
+			    "cookie:%"PRIu32" != stat:%"PRIu32", "
+			    "denying durable reconnect\n",
+			    name,
+			    "st_ex_flags",
+			    cookie_st->st_ex_flags,
+			    fsp_st->st_ex_flags);
 		return false;
 	}
 
@@ -532,7 +532,7 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 	NTSTATUS status;
 	bool ok;
 	int ret;
-	int flags = 0;
+	struct vfs_open_how how = { .flags = 0, };
 	struct file_id file_id;
 	struct smb_filename *smb_fname = NULL;
 	enum ndr_err_code ndr_err;
@@ -804,14 +804,14 @@ NTSTATUS vfs_default_durable_reconnect(struct connection_struct *conn,
 	 * TODO: properly calculate open flags
 	 */
 	if (fsp->fsp_flags.can_write && fsp->fsp_flags.can_read) {
-		flags = O_RDWR;
+		how.flags = O_RDWR;
 	} else if (fsp->fsp_flags.can_write) {
-		flags = O_WRONLY;
+		how.flags = O_WRONLY;
 	} else if (fsp->fsp_flags.can_read) {
-		flags = O_RDONLY;
+		how.flags = O_RDONLY;
 	}
 
-	status = fd_openat(conn->cwd_fsp, fsp->fsp_name, fsp, flags, 0);
+	status = fd_openat(conn->cwd_fsp, fsp->fsp_name, fsp, &how);
 	if (!NT_STATUS_IS_OK(status)) {
 		TALLOC_FREE(lck);
 		DEBUG(1, ("vfs_default_durable_reconnect: failed to open "
