@@ -27,11 +27,11 @@
 #include "conn_tdb.h"
 #include "util_tdb.h"
 #include "lib/util/string_wrappers.h"
+#include "../libcli/security/session.h"
 
 struct connections_forall_state {
 	struct db_context *session_by_pid;
-	int (*fn)(const struct connections_key *key,
-		  const struct connections_data *data,
+	int (*fn)(const struct connections_data *data,
 		  void *private_data);
 	void *private_data;
 	int count;
@@ -44,7 +44,8 @@ struct connections_forall_session {
 	fstring addr;
 	uint16_t cipher;
 	uint16_t dialect;
-	uint8_t signing_flags;
+	uint16_t signing;
+	bool authenticated;
 };
 
 static int collect_sessions_fn(struct smbXsrv_session_global0 *global,
@@ -56,6 +57,7 @@ static int collect_sessions_fn(struct smbXsrv_session_global0 *global,
 
 	uint32_t id = global->session_global_id;
 	struct connections_forall_session sess;
+	enum security_user_level ul;
 
 	if (global->auth_session_info == NULL) {
 		sess.uid = -1;
@@ -67,8 +69,14 @@ static int collect_sessions_fn(struct smbXsrv_session_global0 *global,
 	fstrcpy(sess.machine, global->channels[0].remote_name);
 	fstrcpy(sess.addr, global->channels[0].remote_address);
 	sess.cipher = global->channels[0].encryption_cipher;
+	sess.signing = global->channels[0].signing_algo;
 	sess.dialect = global->connection_dialect;
-	sess.signing_flags = global->signing_flags;
+	ul = security_session_user_level(global->auth_session_info, NULL);
+	if (ul >= SECURITY_USER) {
+		sess.authenticated = true;
+	} else {
+		sess.authenticated = false;
+	}
 
 	status = dbwrap_store(state->session_by_pid,
 			      make_tdb_data((void*)&id, sizeof(id)),
@@ -87,7 +95,6 @@ static int traverse_tcon_fn(struct smbXsrv_tcon_global0 *global,
 	struct connections_forall_state *state =
 		(struct connections_forall_state*)connections_forall_state;
 
-	struct connections_key key;
 	struct connections_data data;
 
 	uint32_t sess_id = global->session_global_id;
@@ -118,30 +125,30 @@ static int traverse_tcon_fn(struct smbXsrv_tcon_global0 *global,
 		memcpy((uint8_t *)&sess, val.dptr, val.dsize);
 	}
 
-	ZERO_STRUCT(key);
 	ZERO_STRUCT(data);
 
-	key.pid = data.pid = global->server_id;
-	key.cnum = data.cnum = global->tcon_global_id;
-	fstrcpy(key.name, global->share_name);
+	data.pid = global->server_id;
+	data.cnum = global->tcon_global_id;
+	data.sess_id = sess_id;
 	fstrcpy(data.servicename, global->share_name);
 	data.uid = sess.uid;
 	data.gid = sess.gid;
 	fstrcpy(data.addr, sess.addr);
 	fstrcpy(data.machine, sess.machine);
-	data.start = nt_time_to_unix(global->creation_time);
+	data.start = global->creation_time;
 	data.encryption_flags = global->encryption_flags;
 	data.cipher = sess.cipher;
 	data.dialect = sess.dialect;
+	data.signing = sess.signing;
 	data.signing_flags = global->signing_flags;
+	data.authenticated = sess.authenticated;
 
 	state->count++;
 
-	return state->fn(&key, &data, state->private_data);
+	return state->fn(&data, state->private_data);
 }
 
-int connections_forall_read(int (*fn)(const struct connections_key *key,
-				      const struct connections_data *data,
+int connections_forall_read(int (*fn)(const struct connections_data *data,
 				      void *private_data),
 			    void *private_data)
 {
