@@ -35,6 +35,7 @@
 #include "system/filesys.h"
 #include "lib/cmdline/cmdline.h"
 #include "lib/param/loadparm.h"
+#include "lib/param/param.h"
 #include "lib/crypto/gnutls_helpers.h"
 #include "cmdline_contexts.h"
 
@@ -192,6 +193,17 @@ static bool do_idmap_check(void)
 			uint32_t maxranges =
 				(c->high - c->low  + 1) / rangesize;
 
+			if (((c->high - c->low  + 1) % rangesize) != 0) {
+				fprintf(stderr,
+					"WARNING: The idmap autorid range "
+					"[%u-%u] SHOULD be a multiple of "
+					"the rangesize [%u]!"
+					"\n\n",
+					c->low,
+					c->high,
+					rangesize);
+			}
+
 			if (maxranges < 2) {
 				fprintf(stderr,
 					"ERROR: The idmap autorid range "
@@ -217,16 +229,21 @@ static bool do_idmap_check(void)
 
 			if ((c->low >= x->low && c->low <= x->high) ||
 			    (c->high >= x->low && c->high <= x->high)) {
-				/* Allow overlapping ranges for idmap_ad */
+				/*
+				 * Allow overlapping ranges for idmap_ad
+				 * and idmap_nss
+				 */
 				ok = strequal(c->backend, x->backend);
 				if (ok) {
-					ok = strequal(c->backend, "ad");
+					ok = strequal(c->backend, "ad") ||
+					     strequal(c->backend, "nss");
 					if (ok) {
 						fprintf(stderr,
-							"NOTE: The idmap_ad "
+							"NOTE: The idmap_%s "
 							"range for the domain "
 							"%s overlaps with the "
 							"range of %s.\n\n",
+							c->backend,
 							c->domain_name,
 							x->domain_name);
 						continue;
@@ -598,11 +615,106 @@ static int do_global_checks(void)
 		ret = 1;
 	}
 
-	if (!lp_server_schannel()) {
+	if (lp_ldap_server_require_strong_auth() ==
+	    LDAP_SERVER_REQUIRE_STRONG_AUTH_ALLOW_SASL_OVER_TLS)
+	{
 		fprintf(stderr,
-			"WARNING: You have configured 'server schannel = no'. "
-			"Your server is vulernable to \"ZeroLogon\" "
-			"(CVE-2020-1472)\n\n");
+			"WARNING: You have not configured "
+			"'ldap server require strong auth = "
+			"allow_sasl_over_tls'.\n"
+			"Please change to 'yes' (preferred) or "
+			"'allow_sasl_without_tls_channel_bindings' "
+			"(if really needed)\n\n");
+	}
+
+	if (lp_server_schannel() != true) { /* can be 'auto' */
+		fprintf(stderr,
+			"WARNING: You have not configured "
+			"'server schannel = yes' (the default). "
+			"Your server is vulnerable to \"ZeroLogon\" "
+			"(CVE-2020-1472)\n"
+			"If required use individual "
+			"'server require schannel:COMPUTERACCOUNT$ = no' "
+			"options\n\n");
+	}
+	if (lp_allow_nt4_crypto()) {
+		fprintf(stderr,
+			"WARNING: You have not configured "
+			"'allow nt4 crypto = no' (the default). "
+			"Your server is vulnerable to "
+			"CVE-2022-38023 and others!\n"
+			"If required use individual "
+			"'allow nt4 crypto:COMPUTERACCOUNT$ = yes' "
+			"options\n\n");
+	}
+	if (!lp_reject_md5_clients()) {
+		fprintf(stderr,
+			"WARNING: You have not configured "
+			"'reject md5 clients = yes' (the default). "
+			"Your server is vulnerable to "
+			"CVE-2022-38023!\n"
+			"If required use individual "
+			"'server reject md5 schannel:COMPUTERACCOUNT$ = yes' "
+			"options\n\n");
+	}
+	if (!lp_server_schannel_require_seal()) {
+		fprintf(stderr,
+			"WARNING: You have not configured "
+			"'server schannel require seal = yes' (the default). "
+			"Your server is vulnerable to "
+			"CVE-2022-38023!\n"
+			"If required use individual "
+			"'server schannel require seal:COMPUTERACCOUNT$ = no' "
+			"options\n\n");
+	}
+
+	if (lp_client_schannel() != true) { /* can be 'auto' */
+		fprintf(stderr,
+			"WARNING: You have not configured "
+			"'client schannel = yes' (the default). "
+			"Your server is vulnerable to \"ZeroLogon\" "
+			"(CVE-2020-1472)\n"
+			"If required use individual "
+			"'client schannel:NETBIOSDOMAIN = no' "
+			"options\n\n");
+	}
+	if (!lp_reject_md5_servers()) {
+		fprintf(stderr,
+			"WARNING: You have not configured "
+			"'reject md5 servers = yes' (the default). "
+			"Your server is vulnerable to "
+			"CVE-2022-38023\n"
+			"If required use individual "
+			"'reject md5 servers:NETBIOSDOMAIN = no' "
+			"options\n\n");
+	}
+	if (!lp_require_strong_key()) {
+		fprintf(stderr,
+			"WARNING: You have not configured "
+			"'require strong key = yes' (the default). "
+			"Your server is vulnerable to "
+			"CVE-2022-38023\n"
+			"If required use individual "
+			"'require strong key:NETBIOSDOMAIN = no' "
+			"options\n\n");
+	}
+	if (!lp_winbind_sealed_pipes()) {
+		fprintf(stderr,
+			"WARNING: You have not configured "
+			"'winbind sealed pipes = yes' (the default). "
+			"Your server is vulnerable to "
+			"CVE-2022-38023\n"
+			"If required use individual "
+			"'winbind sealed pipes:NETBIOSDOMAIN = no' "
+			"options\n\n");
+	}
+
+	if (lp_kerberos_encryption_types() == KERBEROS_ETYPES_LEGACY) {
+		fprintf(stderr,
+			"WARNING: You have configured "
+			"'kerberos encryption types = legacy'. "
+			"Your server is vulnerable to "
+			"CVE-2022-37966\n\n");
 	}
 
 	return ret;
@@ -795,6 +907,7 @@ static void do_per_share_checks(int s)
 	};
 
 	TALLOC_CTX *frame = talloc_stackframe();
+	struct loadparm_context *lp_ctx = NULL;
 
 	smb_init_locale();
 
@@ -806,13 +919,14 @@ static void do_per_share_checks(int s)
 		ret = 1;
 		goto done;
 	}
+	lp_ctx = samba_cmdline_get_lp_ctx();
 
 	/*
 	 * Set the default debug level to 1.
 	 * Allow it to be overridden by the command line,
 	 * not by smb.conf.
 	 */
-	lp_set_cmdline("log level", "1");
+	lpcfg_set_cmdline(lp_ctx, "log level", "1");
 
 	pc = samba_popt_get_context(getprogname(),
 				    argc,
