@@ -1,22 +1,22 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
    LDAP protocol helper functions for SAMBA
-   
+
    Copyright (C) Simo Sorce 2005
-    
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-   
+
 */
 
 #include "includes.h"
@@ -30,9 +30,9 @@
 
 static bool decode_server_sort_response(void *mem_ctx, DATA_BLOB in, void *_out)
 {
-	void **out = (void **)_out;
+	void **out = _out;
 	DATA_BLOB attr;
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	struct ldb_sort_resp_control *lsrc;
 
 	if (!data) return false;
@@ -76,10 +76,10 @@ static bool decode_server_sort_response(void *mem_ctx, DATA_BLOB in, void *_out)
 
 static bool decode_server_sort_request(void *mem_ctx, DATA_BLOB in, void *_out)
 {
-	void **out = (void **)_out;
+	void **out = _out;
 	DATA_BLOB attr;
 	DATA_BLOB rule;
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	struct ldb_server_sort_control **lssc;
 	int num;
 
@@ -135,7 +135,7 @@ static bool decode_server_sort_request(void *mem_ctx, DATA_BLOB in, void *_out)
 			}
 			lssc[num]->reverse = reverse;
 		}
-	
+
 		if (!asn1_end_tag(data)) {
 			return false;
 		}
@@ -156,7 +156,7 @@ static bool decode_server_sort_request(void *mem_ctx, DATA_BLOB in, void *_out)
 
 static bool decode_extended_dn_request(void *mem_ctx, DATA_BLOB in, void *_out)
 {
-	void **out = (void **)_out;
+	void **out = _out;
 	struct asn1_data *data;
 	struct ldb_extended_dn_control *ledc;
 
@@ -166,7 +166,7 @@ static bool decode_extended_dn_request(void *mem_ctx, DATA_BLOB in, void *_out)
 		return true;
 	}
 
-	data = asn1_init(mem_ctx);
+	data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	if (!data) return false;
 
 	if (!asn1_load(data, in)) {
@@ -185,7 +185,7 @@ static bool decode_extended_dn_request(void *mem_ctx, DATA_BLOB in, void *_out)
 	if (!asn1_read_Integer(data, &(ledc->type))) {
 		return false;
 	}
-	
+
 	if (!asn1_end_tag(data)) {
 		return false;
 	}
@@ -197,8 +197,8 @@ static bool decode_extended_dn_request(void *mem_ctx, DATA_BLOB in, void *_out)
 
 static bool decode_sd_flags_request(void *mem_ctx, DATA_BLOB in, void *_out)
 {
-	void **out = (void **)_out;
-	struct asn1_data *data = asn1_init(mem_ctx);
+	void **out = _out;
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	struct ldb_sd_flags_control *lsdfc;
 
 	if (!data) return false;
@@ -231,8 +231,8 @@ static bool decode_sd_flags_request(void *mem_ctx, DATA_BLOB in, void *_out)
 
 static bool decode_search_options_request(void *mem_ctx, DATA_BLOB in, void *_out)
 {
-	void **out = (void **)_out;
-	struct asn1_data *data = asn1_init(mem_ctx);
+	void **out = _out;
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	struct ldb_search_options_control *lsoc;
 
 	if (!data) return false;
@@ -265,9 +265,9 @@ static bool decode_search_options_request(void *mem_ctx, DATA_BLOB in, void *_ou
 
 static bool decode_paged_results_request(void *mem_ctx, DATA_BLOB in, void *_out)
 {
-	void **out = (void **)_out;
+	void **out = _out;
 	DATA_BLOB cookie;
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	struct ldb_paged_control *lprc;
 
 	if (!data) return false;
@@ -288,7 +288,7 @@ static bool decode_paged_results_request(void *mem_ctx, DATA_BLOB in, void *_out
 	if (!asn1_read_Integer(data, &(lprc->size))) {
 		return false;
 	}
-	
+
 	if (!asn1_read_OctetString(data, mem_ctx, &cookie)) {
 		return false;
 	}
@@ -314,9 +314,9 @@ static bool decode_paged_results_request(void *mem_ctx, DATA_BLOB in, void *_out
 
 static bool decode_dirsync_request(void *mem_ctx, DATA_BLOB in, void *_out)
 {
-	void **out = (void **)_out;
+	void **out = _out;
 	DATA_BLOB cookie;
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	struct ldb_dirsync_control *ldc;
 
 	if (!data) return false;
@@ -337,11 +337,11 @@ static bool decode_dirsync_request(void *mem_ctx, DATA_BLOB in, void *_out)
 	if (!asn1_read_Integer(data, &(ldc->flags))) {
 		return false;
 	}
-	
+
 	if (!asn1_read_Integer(data, &(ldc->max_attributes))) {
 		return false;
 	}
-	
+
 	if (!asn1_read_OctetString(data, mem_ctx, &cookie)) {
 		return false;
 	}
@@ -370,9 +370,9 @@ static bool decode_dirsync_request(void *mem_ctx, DATA_BLOB in, void *_out)
  */
 static bool decode_asq_control(void *mem_ctx, DATA_BLOB in, void *_out)
 {
-	void **out = (void **)_out;
+	void **out = _out;
 	DATA_BLOB source_attribute;
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	struct ldb_asq_control *lac;
 
 	if (!data) return false;
@@ -431,9 +431,9 @@ static bool decode_asq_control(void *mem_ctx, DATA_BLOB in, void *_out)
 
 static bool decode_verify_name_request(void *mem_ctx, DATA_BLOB in, void *_out)
 {
-	void **out = (void **)_out;
+	void **out = _out;
 	DATA_BLOB name;
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	struct ldb_verify_name_control *lvnc;
 	int len;
 
@@ -461,10 +461,10 @@ static bool decode_verify_name_request(void *mem_ctx, DATA_BLOB in, void *_out)
 	}
 
 	if (name.length) {
-		len = utf16_len_n(name.data, name.length);
+		len = utf16_null_terminated_len_n(name.data, name.length);
 		convert_string_talloc(mem_ctx, CH_UTF16, CH_UNIX,
 					name.data, len,
-					(void **)&lvnc->gc, &lvnc->gc_len);
+					&lvnc->gc, &lvnc->gc_len);
 
 		if (!(lvnc->gc)) {
 			return false;
@@ -485,7 +485,7 @@ static bool decode_verify_name_request(void *mem_ctx, DATA_BLOB in, void *_out)
 static bool encode_verify_name_request(void *mem_ctx, void *in, DATA_BLOB *out)
 {
 	struct ldb_verify_name_control *lvnc = talloc_get_type(in, struct ldb_verify_name_control);
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	DATA_BLOB gc_utf16;
 
 	if (!data) return false;
@@ -499,9 +499,14 @@ static bool encode_verify_name_request(void *mem_ctx, void *in, DATA_BLOB *out)
 	}
 
 	if (lvnc->gc_len) {
-		convert_string_talloc(mem_ctx, CH_UNIX, CH_UTF16,
-						lvnc->gc, lvnc->gc_len,
-						(void **)&gc_utf16.data, &gc_utf16.length);
+		bool ok;
+
+		ok = convert_string_talloc(mem_ctx, CH_UNIX, CH_UTF16,
+					   lvnc->gc, lvnc->gc_len,
+					   &gc_utf16.data, &gc_utf16.length);
+		if (!ok) {
+			return false;
+		}
 		if (!asn1_write_OctetString(data, gc_utf16.data, gc_utf16.length)) {
 			return false;
 		}
@@ -526,9 +531,9 @@ static bool encode_verify_name_request(void *mem_ctx, void *in, DATA_BLOB *out)
 
 static bool decode_vlv_request(void *mem_ctx, DATA_BLOB in, void *_out)
 {
-	void **out = (void **)_out;
+	void **out = _out;
 	DATA_BLOB assertion_value, context_id;
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	struct ldb_vlv_req_control *lvrc;
 
 	if (!data) return false;
@@ -549,7 +554,7 @@ static bool decode_vlv_request(void *mem_ctx, DATA_BLOB in, void *_out)
 	if (!asn1_read_Integer(data, &(lvrc->beforeCount))) {
 		return false;
 	}
-	
+
 	if (!asn1_read_Integer(data, &(lvrc->afterCount))) {
 		return false;
 	}
@@ -624,9 +629,9 @@ static bool decode_vlv_request(void *mem_ctx, DATA_BLOB in, void *_out)
 
 static bool decode_vlv_response(void *mem_ctx, DATA_BLOB in, void *_out)
 {
-	void **out = (void **)_out;
+	void **out = _out;
 	DATA_BLOB context_id;
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	struct ldb_vlv_resp_control *lvrc;
 
 	if (!data) return false;
@@ -647,11 +652,11 @@ static bool decode_vlv_response(void *mem_ctx, DATA_BLOB in, void *_out)
 	if (!asn1_read_Integer(data, &(lvrc->targetPosition))) {
 		return false;
 	}
-	
+
 	if (!asn1_read_Integer(data, &(lvrc->contentCount))) {
 		return false;
 	}
-	
+
 	if (!asn1_read_enumerated(data, &(lvrc->vlv_result))) {
 		return false;
 	}
@@ -682,7 +687,7 @@ static bool decode_vlv_response(void *mem_ctx, DATA_BLOB in, void *_out)
 static bool encode_server_sort_response(void *mem_ctx, void *in, DATA_BLOB *out)
 {
 	struct ldb_sort_resp_control *lsrc = talloc_get_type(in, struct ldb_sort_resp_control);
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 
 	if (!data) return false;
 
@@ -716,7 +721,7 @@ static bool encode_server_sort_response(void *mem_ctx, void *in, DATA_BLOB *out)
 static bool encode_server_sort_request(void *mem_ctx, void *in, DATA_BLOB *out)
 {
 	struct ldb_server_sort_control **lssc = talloc_get_type(in, struct ldb_server_sort_control *);
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	int num;
 
 	if (!data) return false;
@@ -736,7 +741,7 @@ static bool encode_server_sort_request(void *mem_ctx, void *in, DATA_BLOB *out)
 		if (!asn1_push_tag(data, ASN1_SEQUENCE(0))) {
 			return false;
 		}
-		
+
 		if (!asn1_write_OctetString(data, lssc[num]->attributeName, strlen(lssc[num]->attributeName))) {
 			return false;
 		}
@@ -782,7 +787,7 @@ static bool encode_extended_dn_request(void *mem_ctx, void *in, DATA_BLOB *out)
 		return true;
 	}
 
-	data = asn1_init(mem_ctx);
+	data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 
 	if (!data) return false;
 
@@ -810,7 +815,7 @@ static bool encode_extended_dn_request(void *mem_ctx, void *in, DATA_BLOB *out)
 static bool encode_sd_flags_request(void *mem_ctx, void *in, DATA_BLOB *out)
 {
 	struct ldb_sd_flags_control *lsdfc = talloc_get_type(in, struct ldb_sd_flags_control);
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 
 	if (!data) return false;
 
@@ -838,7 +843,7 @@ static bool encode_sd_flags_request(void *mem_ctx, void *in, DATA_BLOB *out)
 static bool encode_search_options_request(void *mem_ctx, void *in, DATA_BLOB *out)
 {
 	struct ldb_search_options_control *lsoc = talloc_get_type(in, struct ldb_search_options_control);
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 
 	if (!data) return false;
 
@@ -866,7 +871,7 @@ static bool encode_search_options_request(void *mem_ctx, void *in, DATA_BLOB *ou
 static bool encode_paged_results_request(void *mem_ctx, void *in, DATA_BLOB *out)
 {
 	struct ldb_paged_control *lprc = talloc_get_type(in, struct ldb_paged_control);
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 
 	if (!data) return false;
 
@@ -880,7 +885,7 @@ static bool encode_paged_results_request(void *mem_ctx, void *in, DATA_BLOB *out
 
 	if (!asn1_write_OctetString(data, lprc->cookie, lprc->cookie_len)) {
 		return false;
-	}	
+	}
 
 	if (!asn1_pop_tag(data)) {
 		return false;
@@ -901,7 +906,7 @@ static bool encode_paged_results_request(void *mem_ctx, void *in, DATA_BLOB *out
 static bool encode_asq_control(void *mem_ctx, void *in, DATA_BLOB *out)
 {
 	struct ldb_asq_control *lac = talloc_get_type(in, struct ldb_asq_control);
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 
 	if (!data) return false;
 
@@ -936,7 +941,7 @@ static bool encode_asq_control(void *mem_ctx, void *in, DATA_BLOB *out)
 static bool encode_dirsync_request(void *mem_ctx, void *in, DATA_BLOB *out)
 {
 	struct ldb_dirsync_control *ldc = talloc_get_type(in, struct ldb_dirsync_control);
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 
 	if (!data) return false;
 
@@ -954,7 +959,7 @@ static bool encode_dirsync_request(void *mem_ctx, void *in, DATA_BLOB *out)
 
 	if (!asn1_write_OctetString(data, ldc->cookie, ldc->cookie_len)) {
 		return false;
-	}	
+	}
 
 	if (!asn1_pop_tag(data)) {
 		return false;
@@ -972,7 +977,7 @@ static bool encode_dirsync_request(void *mem_ctx, void *in, DATA_BLOB *out)
 static bool encode_vlv_request(void *mem_ctx, void *in, DATA_BLOB *out)
 {
 	struct ldb_vlv_req_control *lvrc = talloc_get_type(in, struct ldb_vlv_req_control);
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 
 	if (!data) return false;
 
@@ -992,7 +997,7 @@ static bool encode_vlv_request(void *mem_ctx, void *in, DATA_BLOB *out)
 		if (!asn1_push_tag(data, ASN1_CONTEXT(0))) {
 			return false;
 		}
-		
+
 		if (!asn1_write_Integer(data, lvrc->match.byOffset.offset)) {
 			return false;
 		}
@@ -1008,7 +1013,7 @@ static bool encode_vlv_request(void *mem_ctx, void *in, DATA_BLOB *out)
 		if (!asn1_push_tag(data, ASN1_CONTEXT_SIMPLE(1))) {
 			return false;
 		}
-		
+
 		if (!asn1_write(data, lvrc->match.gtOrEq.value, lvrc->match.gtOrEq.value_len)) {
 			return false;
 		}
@@ -1040,7 +1045,7 @@ static bool encode_vlv_request(void *mem_ctx, void *in, DATA_BLOB *out)
 static bool encode_vlv_response(void *mem_ctx, void *in, DATA_BLOB *out)
 {
 	struct ldb_vlv_resp_control *lvrc = talloc_get_type(in, struct ldb_vlv_resp_control);
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 
 	if (!data) return false;
 
@@ -1083,16 +1088,16 @@ static bool encode_openldap_dereference(void *mem_ctx, void *in, DATA_BLOB *out)
 {
 	struct dsdb_openldap_dereference_control *control = talloc_get_type(in, struct dsdb_openldap_dereference_control);
 	int i,j;
-	struct asn1_data *data = asn1_init(mem_ctx);
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 
 	if (!data) return false;
-	
+
 	if (!control) return false;
-	
+
 	if (!asn1_push_tag(data, ASN1_SEQUENCE(0))) {
 		return false;
 	}
-	
+
 	for (i=0; control->dereference && control->dereference[i]; i++) {
 		if (!asn1_push_tag(data, ASN1_SEQUENCE(0))) {
 			return false;
@@ -1104,12 +1109,12 @@ static bool encode_openldap_dereference(void *mem_ctx, void *in, DATA_BLOB *out)
 			return false;
 		}
 		for (j=0; control->dereference && control->dereference[i]->dereference_attribute[j]; j++) {
-			if (!asn1_write_OctetString(data, control->dereference[i]->dereference_attribute[j], 
+			if (!asn1_write_OctetString(data, control->dereference[i]->dereference_attribute[j],
 						    strlen(control->dereference[i]->dereference_attribute[j]))) {
 				return false;
 			}
 		}
-		
+
 		if (!asn1_pop_tag(data)) {
 			return false;
 		}
@@ -1131,8 +1136,8 @@ static bool encode_openldap_dereference(void *mem_ctx, void *in, DATA_BLOB *out)
 
 static bool decode_openldap_dereference(void *mem_ctx, DATA_BLOB in, void *_out)
 {
-	void **out = (void **)_out;
-	struct asn1_data *data = asn1_init(mem_ctx);
+	void **out = _out;
+	struct asn1_data *data = asn1_init(mem_ctx, ASN1_MAX_TREE_DEPTH);
 	struct dsdb_openldap_dereference_result_control *control;
 	struct dsdb_openldap_dereference_result **r = NULL;
 	int i = 0;
@@ -1154,7 +1159,7 @@ static bool decode_openldap_dereference(void *mem_ctx, DATA_BLOB in, void *_out)
 		return false;
 	}
 
-	while (asn1_tag_remaining(data) > 0) {					
+	while (asn1_tag_remaining(data) > 0) {
 		r = talloc_realloc(control, r, struct dsdb_openldap_dereference_result *, i + 2);
 		if (!r) {
 			return false;
@@ -1167,7 +1172,7 @@ static bool decode_openldap_dereference(void *mem_ctx, DATA_BLOB in, void *_out)
 		if (!asn1_start_tag(data, ASN1_SEQUENCE(0))) {
 			return false;
 		}
-		
+
 		if (!asn1_read_OctetString_talloc(r[i], data, &r[i]->source_attribute)) {
 			return false;
 		}
@@ -1261,7 +1266,8 @@ static const struct ldap_control_handler ldap_known_controls[] = {
 	{ LDB_CONTROL_AS_SYSTEM_OID, NULL, NULL },
 	{ DSDB_CONTROL_PASSWORD_CHANGE_STATUS_OID, NULL, NULL },
 	{ DSDB_CONTROL_PASSWORD_HASH_VALUES_OID, NULL, NULL },
-	{ DSDB_CONTROL_PASSWORD_CHANGE_OID, NULL, NULL },
+	{ DSDB_CONTROL_PASSWORD_CHANGE_OLD_PW_CHECKED_OID, NULL, NULL },
+	{ DSDB_CONTROL_PASSWORD_ACL_VALIDATION_OID, NULL, NULL },
 	{ DSDB_CONTROL_APPLY_LINKS, NULL, NULL },
 	{ LDB_CONTROL_BYPASS_OPERATIONAL_OID, NULL, NULL },
 	{ DSDB_CONTROL_CHANGEREPLMETADATA_OID, NULL, NULL },
@@ -1271,6 +1277,8 @@ static const struct ldap_control_handler ldap_known_controls[] = {
 	{ DSDB_EXTENDED_ALLOCATE_RID_POOL, NULL, NULL },
 	{ DSDB_CONTROL_NO_GLOBAL_CATALOG, NULL, NULL },
 	{ DSDB_EXTENDED_SCHEMA_UPGRADE_IN_PROGRESS_OID, NULL, NULL },
+	{ DSDB_CONTROL_TRANSACTION_IDENTIFIER_OID, NULL, NULL},
+	{ DSDB_CONTROL_CALCULATED_DEFAULT_SD_OID, NULL, NULL },
 	{ NULL, NULL, NULL }
 };
 
@@ -1278,4 +1286,3 @@ const struct ldap_control_handler *samba_ldap_control_handlers(void)
 {
 	return ldap_known_controls;
 }
-
